-------------------------------------------------------------------------------
-- MrTrader 
--
-- Lightweight addon to access profession information of alts.
--
-- Send suggestions, comments, and bugs to user@zedonline.net
-------------------------------------------------------------------------------

MrTraderSkillWindow = {};

local NUM_FILTERS_TO_DISPLAY = 18;
local TRADE_SKILLS_DISPLAYED = 11;
local TRADE_SKILL_TEXT_WIDTH = 275;
MRTRADER_TRADE_SKILL_HEIGHT = 16;

MRTRADER_TRADE_SKILL_FILTER_HEIGHT = 20;

-- Coloring for trade skills
local TradeSkillTypeColor = { };
TradeSkillTypeColor["optimal"]	= { r = 1.00, g = 0.50, b = 0.25,	font = GameFontNormalLeftOrange };
TradeSkillTypeColor["medium"]	= { r = 1.00, g = 1.00, b = 0.00,	font = GameFontNormalLeftYellow };
TradeSkillTypeColor["easy"]		= { r = 0.25, g = 0.75, b = 0.25,	font = GameFontNormalLeftLightGreen };
TradeSkillTypeColor["trivial"]	= { r = 0.50, g = 0.50, b = 0.50,	font = GameFontNormalLeftGrey };
TradeSkillTypeColor["header"]	= { r = 1.00, g = 0.82, b = 0,		font = GameFontNormalLeft };

-- Colorblind Support
local TradeSkillTypePrefix = {
["optimal"] = " [+++] ",
["medium"] = " [++] ",
["easy"] = " [+] ",
["trivial"] = " ", 
["header"] = " ",
}

UIPanelWindows["MrTrader_SkillFrame"] =	{ area = "left", pushable = 3 };

local L = LibStub("AceLocale-3.0"):GetLocale("MrTrader", true);

function MrTraderSkillWindow.Show()
	MrTraderSkillWindow.filters = {};
	MrTraderSkillWindow.HideBlizzardFrame();
	ShowUIPanel(MrTrader_SkillFrame);
	SetPortraitTexture(MrTrader_SkillFramePortrait, "player");
	
	FauxScrollFrame_SetOffset(MrTrader_SkillListScrollFrame, 0);
	MrTrader_SkillListScrollFrameScrollBar:SetMinMaxValues(0, 0); 
	MrTrader_SkillListScrollFrameScrollBar:SetValue(0);
	
	MrTraderSkillWindow.PopulateFilterTree();
	
	MrTraderSkillWindow_OnSearchTextChange(MrTrader_SkillFrameEditBox);
	
	MrTraderSkillWindow.selectedFilter = "All Items";
	MrTraderSkillWindow.selectedIndex = 0;
	SetTradeSkillSubClassFilter(MrTraderSkillWindow.selectedIndex, 1, 1);
	
	MrTraderSkillWindow.SetSkillSelection(nil, 0);
	
	MrTraderSkillWindow.Update();
end

function MrTraderSkillWindow.PopulateFilterTree()
	MrTraderSkillWindow.filters = {};
	
	tinsert(MrTraderSkillWindow.filters, 
		{
			title = L["All Items"],
			filterType = "all",
			subgroups = {},
		});
	tinsert(MrTraderSkillWindow.filters, 
		{
			title = L["Categories"],
			subgroups = {},
		});
	tinsert(MrTraderSkillWindow.filters, 
		{
			title = L["Inventory Slots"],
			subgroups = {},
		});
		
	local groups = {GetTradeSkillSubClasses()};
	local numGroups = getn(groups);
	local parent = MrTraderSkillWindow.filters[2];
	for i=1, numGroups do
		tinsert(parent.subgroups, 
			{
				title = groups[i],
				filterType = "category",
				filterIndex = i,
			});
	end
	
	groups = {GetTradeSkillInvSlots()};
	numGroups = getn(groups);
	local parent = MrTraderSkillWindow.filters[3];
	for i=1, numGroups do
		tinsert(parent.subgroups, 
			{
				title = groups[i],
				filterType = "invslot",
				filterIndex = i,
			});
	end	
	
	MrTraderSkillWindow.filterCategory = MrTraderSkillWindow.filters[1];
	MrTraderSkillWindow.filterSelection = MrTraderSkillWindow.filters[1];
end

function MrTraderSkillWindow.Close()
	HideUIPanel(MrTrader_SkillFrame);
end

function MrTraderSkillWindow.Update()
	if( TradeSkillFrame and TradeSkillFrame:IsVisible() ) then
		MrTraderSkillWindow.HideBlizzardFrame();
	end
	
	-- Update Display
	MrTraderSkillWindow.UpdateHeader();
	MrTraderSkillWindow.UpdateCategories();
	MrTraderSkillWindow.UpdateSkillList();

end

function MrTraderSkillWindow.UpdateHeader()
	local name, rank, maxRank = GetTradeSkillLine();

	-- Title
	MrTrader_SkillFrameTitleText:SetFormattedText(TRADE_SKILL_TITLE, name);
	
	-- Link Button
	if ( IsTradeSkillLinked() ) then
		MrTrader_SkillLinkButton:Hide();
	else
		if ( GetTradeSkillListLink() ) then
			MrTrader_SkillLinkButton:Show();
		else
			MrTrader_SkillLinkButton:Hide();
		end
	end
	
	-- Rank Bar
	MrTrader_SkillRankFrame:SetStatusBarColor(0.0, 0.0, 1.0, 0.5);
	MrTrader_SkillRankFrameBackground:SetVertexColor(0.0, 0.0, 0.75, 0.5);
	MrTrader_SkillRankFrame:SetMinMaxValues(0, maxRank);
	MrTrader_SkillRankFrame:SetValue(rank);
	MrTrader_SkillRankFrameSkillRank:SetText(rank.."/"..maxRank);
	
	-- Search Bar
	if ( rank < 75 ) and ( not IsTradeSkillLinked() ) then
		MrTrader_SkillFrameEditBox:Hide();
		SetTradeSkillItemNameFilter("");	--In case they are switching from an inspect WITH a filter directly to their own without.
	else
		MrTrader_SkillFrameEditBox:Show();
	end
end

function MrTraderSkillWindow.GetNumTradeSkills()
	local serverCount = GetNumTradeSkills();
	local localCount = 0;
	
	for i=1, serverCount do
		local _, skillType = GetTradeSkillInfo(i);
		if( skillType ~= "header" ) then
			localCount = localCount + 1;
		end
	end
	
	return localCount, serverCount;
end

function MrTraderSkillWindow.GetIndex(skillOffset)
	local serverCount = GetNumTradeSkills();
	local calcOffset = 0;

	if( skillOffset == 0 ) then
		return 0;
	end

	for i=1, serverCount do
		local _, skillType = GetTradeSkillInfo(i);
		if( skillType ~= "header" ) then
			calcOffset = calcOffset + 1;
			if( calcOffset == skillOffset ) then
				return i;
			end
		end	
	end
	
	return 0;
end

function MrTraderSkillWindow.UpdateSkillList()
	local numTradeSkills, totalTradeSkills = MrTraderSkillWindow.GetNumTradeSkills();

	-- If we have no filter of any kind, then we need to empty the skill list
	if( MrTraderSkillWindow.filterSelection == nil ) then
		totalTradeSkills = 0;
		numTradeSkills = 0;
	end

	-- ScrollFrame update
	FauxScrollFrame_Update(MrTrader_SkillListScrollFrame, numTradeSkills, TRADE_SKILLS_DISPLAYED, MRTRADER_TRADE_SKILL_HEIGHT, nil, nil, nil, MrTrader_SkillHighlightFrame, 293, 316 );
	local skillIndex = MrTraderSkillWindow.GetIndex(FauxScrollFrame_GetOffset(MrTrader_SkillListScrollFrame));
	local skillHighlighted = false;
	
	for i=1, TRADE_SKILLS_DISPLAYED do
		local skillName, skillType, numAvailable;
		
		repeat 
			skillIndex = skillIndex + 1;
			skillName, skillType, numAvailable = GetTradeSkillInfo(skillIndex);
		until skillType ~= "header";
		
		skillButton = _G["MrTrader_SkillButton"..i];
		skillButtonText = _G["MrTrader_SkillButton"..i.."Text"];
		skillButtonCount = _G["MrTrader_SkillButton"..i.."Count"];
		if( skillIndex <= totalTradeSkills ) then
			skillButton:SetWidth(323);
			
			local color = TradeSkillTypeColor[skillType];
			if( color ) then
				skillButton:SetNormalFontObject(color.font);
				skillButtonCount:SetVertexColor(color.r, color.g, color.b);
				skillButton.r = color.r;
				skillButton.g = color.g;
				skillButton.b = color.b;
			end
			
			local skillNamePrefix = "";
			local skillNameSuffix = "";
			if ( ENABLE_COLORBLIND_MODE == "1" ) then
				skillNamePrefix = TradeSkillTypePrefix[skillType] or " ";
			end
			
			skillButton:SetID(skillIndex);
			skillButton:Show();
			
			skillButton:SetNormalTexture("");
			_G["MrTrader_SkillButton"..i.."Highlight"]:SetTexture("");
			
			if( numAvailable > 0 ) then
				skillButtonCount:SetText("["..numAvailable.."]");
				MrTrader_SkillFrameDummyString:SetText(skillName);
				nameWidth = MrTrader_SkillFrameDummyString:GetWidth();
				countWidth = skillButtonCount:GetWidth();
				skillButtonText:SetText(skillName);
				if ( nameWidth + 2 + countWidth > TRADE_SKILL_TEXT_WIDTH ) then
					skillButtonText:SetWidth(TRADE_SKILL_TEXT_WIDTH-2-countWidth);
				else
					skillButtonText:SetWidth(0);
				end
			else
				skillButton:SetText(skillNamePrefix..skillName..skillNameSuffix);
				skillButtonText:SetWidth(TRADE_SKILL_TEXT_WIDTH);
				skillButtonCount:SetText(skillCountPrefix);	
			end
			
			-- Place the highlight and lock the highlight state
			if ( MrTraderSkillWindow.selectedSkill == skillIndex ) then
				MrTrader_SkillHighlightFrame:SetPoint("TOPLEFT", "MrTrader_SkillButton"..i, "TOPLEFT", 0, 0);
				MrTrader_SkillHighlightFrame:Show();
				skillButtonCount:SetVertexColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
				skillButton:LockHighlight();
				skillButton.isHighlighted = true;
				skillHighlighted = true;
			else
				skillButton:UnlockHighlight();
				skillButton.isHighlighted = false;
			end
		else
			skillButton:Hide();
		end
	end
	
	if( skillHighlighted == false ) then
		MrTrader_SkillHighlightFrame:Hide();
	end
end

function MrTraderSkillWindow.UpdateCategories()
	local subClasses = {GetTradeSkillSubClasses()};
	local numCategories = getn(subClasses);

	if(MrTraderSkillWindow.filters == nil) then
		return;
	end
	
	-- Initialize the list of open filters
	OPEN_FILTER_LIST = {};

	local numGroups = getn(MrTraderSkillWindow.filters);
	for i=1, numGroups do
		local item = MrTraderSkillWindow.filters[i];
		if( MrTraderSkillWindow.filterCategory == item ) then
			tinsert(OPEN_FILTER_LIST, {item.title, "class", item, true});
			local numSubgroups = getn(item.subgroups);
			for j=1, numSubgroups do
				local subgroup = item.subgroups[j];
				if( MrTraderSkillWindow.filterSelection == subgroup ) then
					tinsert(OPEN_FILTER_LIST, {subgroup.title, "subclass", subgroup, true});			
				else
					tinsert(OPEN_FILTER_LIST, {subgroup.title, "subclass", subgroup, nil});							
				end
			end
		else
			tinsert(OPEN_FILTER_LIST, {item.title, "class", item, nil});			
		end
	end
	
	-- ScrollFrame update
	FauxScrollFrame_Update(MrTrader_SkillFilterScrollFrame, getn(OPEN_FILTER_LIST), NUM_FILTERS_TO_DISPLAY, MRTRADER_TRADE_SKILL_FILTER_HEIGHT);
	local filterIndex = FauxScrollFrame_GetOffset(MrTrader_SkillFilterScrollFrame) ;
	
	-- Display the list of open filters
	local button, index, info, isLast;
	index = FauxScrollFrame_GetOffset(MrTrader_SkillFilterScrollFrame);
	for i=1, NUM_FILTERS_TO_DISPLAY do
		button = _G["MrTrader_SkillFilterButton"..i];
		if ( getn(OPEN_FILTER_LIST) > NUM_FILTERS_TO_DISPLAY ) then
			button:SetWidth(136);
		else
			button:SetWidth(156);
		end

		index = index + 1;
		if ( index <= getn(OPEN_FILTER_LIST) ) then
			info = OPEN_FILTER_LIST[index];
			if ( info ) then
				MrTraderSkillWindow.FilterButton_SetType(button, info[2], info[1], info[5]);
				button.filterRef = info[3];
				--button:SetID(info[3]);
				if ( info[4] ) then
					button:LockHighlight();
				else
					button:UnlockHighlight();
				end
				button:Show();
			end
		else
			button:Hide();
		end
	end
end

function MrTraderSkillWindow.FilterButton_SetType(button, type, text, isLast)
	local normalText = _G[button:GetName().."NormalText"];
	local normalTexture = _G[button:GetName().."NormalTexture"];
	local line = _G[button:GetName().."Lines"];
	
	line:Hide();
	if ( type == "class" ) then
		button:SetText(text);
		normalText:SetPoint("LEFT", button, "LEFT", 4, 0);
		normalTexture:SetAlpha(1.0);	
	elseif ( type == "subclass" ) then
		button:SetText(HIGHLIGHT_FONT_COLOR_CODE..text..FONT_COLOR_CODE_CLOSE);
		normalText:SetPoint("LEFT", button, "LEFT", 12, 0);
		normalTexture:SetAlpha(0.4);
	end
	button.type = type; 
end

function MrTraderSkillWindow.OnSkillSelect(skill, button)
	-- Do something here... maybe
	if ( button == "LeftButton" ) then
		MrTraderSkillWindow.SetSkillSelection(skill, skill:GetID());
		MrTraderSkillWindow.Update();
	end
end

function MrTraderSkillWindow.SetSkillSelection(skillButton, skillID)
	local skillName, skillType, numAvailable, isExpanded, altVerb = GetTradeSkillInfo(skillID);
	local craftable = 1;
	if( not skillName ) then
		craftable = nil;
	end
	
	MrTraderSkillWindow.selectedSkillButton = skillButton;
	MrTraderSkillWindow.selectedSkill = skillID;
	
	if ( MrTraderSkillWindow.selectedSkill > GetNumTradeSkills() ) then
		return;
	end
	
	-- Hide inappropriate controls when no skill is selected
	if ( MrTraderSkillWindow.selectedSkill == 0 ) then
		MrTrader_SkillCreateButton:Hide();
		MrTrader_SkillCreateAllButton:Hide();
		MrTrader_SkillDecrementButton:Hide();
		MrTrader_SkillInputBox:Hide();
		MrTrader_SkillIncrementButton:Hide();
		MrTrader_DetailScrollFrame:Hide();
		return;
	else
		MrTrader_DetailScrollFrame:Show();
	end
	
	local color = TradeSkillTypeColor[skillType];
	if ( color ) then
		MrTrader_SkillHighlight:SetVertexColor(color.r, color.g, color.b);
	end
	
	-- Name and Cooldown
	MrTrader_SkillName:SetText(skillName);
	if ( GetTradeSkillCooldown(skillID) ) then
		MrTrader_SkillCooldown:SetText(COOLDOWN_REMAINING.." "..SecondsToTime(GetTradeSkillCooldown(skillID)));
	else
		MrTrader_SkillCooldown:SetText("");
	end
	
	-- Icon and Number Made
	MrTrader_SkillIcon:SetNormalTexture(GetTradeSkillIcon(skillID));
	local minMade, maxMade = GetTradeSkillNumMade(skillID);
	if ( maxMade > 1 ) then
		if ( minMade == maxMade ) then
			MrTrader_SkillIconCount:SetText(minMade);
		else
			MrTrader_SkillIconCount:SetText(minMade.."-"..maxMade);
		end
		if ( MrTrader_SkillIconCount:GetWidth() > 39 ) then
			MrTrader_SkillIconCount:SetText("~"..floor((minMade + maxMade)/2));
		end
	else
		MrTrader_SkillIconCount:SetText("");
	end
	
	-- Reagents
	local numReagents = GetTradeSkillNumReagents(skillID);
	if(numReagents > 0) then
		MrTrader_SkillReagentLabel:Show();
	else
		MrTrader_SkillReagentLabel:Hide();
	end
	for i=1, numReagents, 1 do
		local reagentName, reagentTexture, reagentCount, playerReagentCount = GetTradeSkillReagentInfo(skillID, i);
		local reagent = _G["MrTrader_SkillReagent"..i]
		local name = _G["MrTrader_SkillReagent"..i.."Name"];
		local count = _G["MrTrader_SkillReagent"..i.."Count"];
		if ( not reagentName or not reagentTexture ) then
			reagent:Hide();
		else
			reagent:Show();
			SetItemButtonTexture(reagent, reagentTexture);
			name:SetText(reagentName);
			-- Grayout items
			if ( playerReagentCount < reagentCount ) then
				SetItemButtonTextureVertexColor(reagent, 0.5, 0.5, 0.5);
				name:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
				craftable = nil;
			else
				SetItemButtonTextureVertexColor(reagent, 1.0, 1.0, 1.0);
				name:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			end
			if ( playerReagentCount >= 100 ) then
				playerReagentCount = "*";
			end
			count:SetText(playerReagentCount.." /"..reagentCount);
		end
	end
	-- Place reagent label
	local reagentToAnchorTo = numReagents;
	if ( (numReagents > 0) and (mod(numReagents, 2) == 0) ) then
		reagentToAnchorTo = reagentToAnchorTo - 1;
	end
	-- Hide unused reagents
	for i=numReagents + 1, MAX_TRADE_SKILL_REAGENTS, 1 do
		_G["MrTrader_SkillReagent"..i]:Hide();
	end
	
	-- Tools
	local spellFocus = BuildColoredListString(GetTradeSkillTools(skillID));
	if ( spellFocus ) then
		MrTrader_SkillRequirementLabel:Show();
		MrTrader_SkillRequirementText:SetText(spellFocus);
	else
		MrTrader_SkillRequirementLabel:Hide();
		MrTrader_SkillRequirementText:SetText("");
	end
	
	-- Description
	if ( GetTradeSkillDescription(skillID) ) then
		MrTrader_SkillDescription:SetText(GetTradeSkillDescription(skillID))
		MrTrader_SkillReagentLabel:SetPoint("TOPLEFT", "MrTrader_SkillDescription", "BOTTOMLEFT", 0, -10);
	else
		MrTrader_SkillDescription:SetText(" ");
		MrTrader_SkillReagentLabel:SetPoint("TOPLEFT", "MrTrader_SkillDescription", "TOPLEFT", 0, 0);
	end
	
	-- Reset the number of items to be created
	MrTrader_SkillInputBox:SetNumber(GetTradeskillRepeatCount());
	
	--Hide inapplicable buttons if we are inspecting. Otherwise show them
	if ( IsTradeSkillLinked() ) then
		MrTrader_SkillCreateButton:Hide();
		MrTrader_SkillCreateAllButton:Hide();
		MrTrader_SkillDecrementButton:Hide();
		MrTrader_SkillInputBox:Hide();
		MrTrader_SkillIncrementButton:Hide();
	else
		--Change button names and show/hide them depending on if this tradeskill creates an item or casts something
		if ( not altVerb ) then
			--Its an item with 'Create'
			MrTrader_SkillCreateAllButton:Show();
			MrTrader_SkillDecrementButton:Show();
			MrTrader_SkillInputBox:Show();
			MrTrader_SkillIncrementButton:Show();
		else
			--Its something else
			MrTrader_SkillCreateAllButton:Hide();
			MrTrader_SkillDecrementButton:Hide();
			MrTrader_SkillInputBox:Hide();
			MrTrader_SkillIncrementButton:Hide();
		end
		
		MrTrader_SkillCreateButton:SetText(altVerb or CREATE);
		MrTrader_SkillCreateButton:Show();
    end

	MrTraderSkillWindow.numAvailable = numAvailable;
end

function MrTraderSkillWindow.OnFilterSelect(filter, button)
	-- Do Something here... maybe
	if( button == "LeftButton" ) then
		if( filter.filterRef == nil ) then
			return;
		end
		
		local item = filter.filterRef;
	
		-- Don't change anything if they clicked on the parent category
		if( item == MrTraderSkillWindow.filterCategory ) then
			return;
		end
		
		MrTraderSkillWindow.filterSelection = nil;
		if( item.subgroups ~= nil ) then
			MrTraderSkillWindow.filterCategory = item;
		end

		if( item.filterType ~= nil ) then
			MrTraderSkillWindow.filterSelection = item;
		end
		
		local subclassFilter = 0;
		local invslotFilter = 0;
		
		if( MrTraderSkillWindow.filterSelection ~= nil ) then
			if( MrTraderSkillWindow.filterSelection.filterType == "invslot" ) then
				invslotFilter = MrTraderSkillWindow.filterSelection.filterIndex;
			elseif( MrTraderSkillWindow.filterSelection.filterType == "category" ) then
				subclassFilter = MrTraderSkillWindow.filterSelection.filterIndex;
			end
		end
		
		SetTradeSkillSubClassFilter(subclassFilter, 1, 1);
		SetTradeSkillInvSlotFilter(invslotFilter, 1, 1)
		
		MrTraderSkillWindow.SetSkillSelection(nil, 0);
		MrTraderSkillWindow.Update();
	end
end

function MrTraderSkillWindow.OnEnterSkillIcon(icon)
	if ( MrTraderSkillWindow.selectedSkill ~= 0 ) then
		GameTooltip:SetOwner(icon, "ANCHOR_RIGHT");
		GameTooltip:SetTradeSkillItem(MrTraderSkillWindow.selectedSkill);
	end
	CursorUpdate(icon);
end

function MrTraderSkillWindow_Update()
	MrTraderSkillWindow.UpdateSkillList();
end

function MrTraderSkillWindow_UpdateFilters()
	MrTraderSkillWindow.UpdateCategories();
end

function MrTraderSkillWindow_UpdateHeaders()
	MrTraderSkillWindow.UpdateHeader();
end

function MrTraderSkillWindow.OnShow()
	MrTraderSkillWindow.HideBlizzardFrame();
end

function MrTraderSkillWindow.OnHide()

end

function MrTraderSkillWindow_OnEnterSkillIcon(icon)
	MrTraderSkillWindow.OnEnterSkillIcon(icon);
end

function MrTraderSkillWindow_OnSearchTextChange(field)
	TradeSkillFilter_OnTextChanged(field);
	
	MrTraderSkillWindow.UpdateSkillList();
end

function MrTraderSkillWindow.OnDecrementClick()
	if ( MrTrader_SkillInputBox:GetNumber() > 0 ) then
		MrTrader_SkillInputBox:SetNumber(MrTrader_SkillInputBox:GetNumber() - 1);
	end
end

function MrTraderSkillWindow.OnIncrementClick()
	if ( MrTrader_SkillInputBox:GetNumber() < 100 ) then
		MrTrader_SkillInputBox:SetNumber(MrTrader_SkillInputBox:GetNumber() + 1);
	end
end

function MrTraderSkillWindow.HideBlizzardFrame()
	local frame = TradeSkillFrame;
	MrTraderSkillWindow.blizzSettings = {};
	
	MrTraderSkillWindow.blizzSettings["strata"] = frame:GetFrameStrata()
	MrTraderSkillWindow.blizzSettings["alpha"]  = frame:GetAlpha()
	MrTraderSkillWindow.blizzSettings["width"]  = frame:GetWidth()
	MrTraderSkillWindow.blizzSettings["height"]  = frame:GetHeight()

	frame:SetAlpha(0)
	frame:SetFrameStrata("BACKGROUND")
	frame:SetPoint("TOPLEFT", SkilletFrame, "TOPLEFT", 5, -5)
	frame:SetWidth(5)
	frame:SetHeight(5)
end

function MrTraderSkillWindow.ShowBlizzardFrame()
	local frame = TradeSkillFrame;
	local settings = MrTraderSkillWindow.blizzSettings;

	if settings["alpha"] then
    	frame:SetAlpha(settings["alpha"])
	end
	if settings["strata"] then
    	frame:SetFrameStrata(settings["strata"])
	end
	if settings["width"] then
    	frame:SetWidth(settings["width"])
	end
	if settings["height"] then
    	frame:SetHeight(settings["height"])
	end
	frame:ClearAllPoints()
end